@@ -19,7 +19,8 @@ def get_username() -> str:
         user = os.environ.get(name)
         if user:
             return user
-    raise ValueError('No username was specified')
+    import pwd
+    return pwd.getpwuid(os.getuid())[0]
 
 
 def get_config(file_name):
@@ -65,7 +66,7 @@ def initialize_proxies() -> Optional[Dict[str, str]]:
         return {}
 
 
-__version__ = '0.23.1'
+__version__ = '0.23.2'
 hostname = get_hostname()
 username = get_username()
 hexss_dir = get_hexss_dir()
